@@ -29,7 +29,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "include/forms.h"
@@ -58,7 +58,6 @@ static char *fl_app_name,
 static void fli_init_resources( void );
 static void fli_set_debug_level( int );
 
-
 /* Command line options */
 
 static XrmOptionDescRec copt[ ] =
@@ -85,7 +84,6 @@ static XrmOptionDescRec copt[ ] =
     { "-ggamma",    "*ggamma",           XrmoptionSepArg, ( caddr_t ) 0   },
 #endif
 
-/*  { "-single",    "*doubleBuffer",     XrmoptionNoArg, ( caddr_t ) "0"   }, */
     { "-bs",        "*backingStore",     XrmoptionNoArg, ( caddr_t ) "0"   }
 };
 
@@ -235,7 +233,7 @@ fl_set_defaults( unsigned long   mask,
     if ( mask & FL_PDVisual )
     {
         SetMember( vclass );
-        strcpy( fli_cntl.vname, fl_vclass_name( cntl->vclass ) );
+        strcpy( fli_cntl.vname, fli_vclass_name( cntl->vclass ) );
     }
 
     if ( mask & FL_PDButtonFontSize )
@@ -312,22 +310,6 @@ fl_set_defaults( unsigned long   mask,
 }
 
 
-/***************************************
- ***************************************/
-
-void
-fl_get_defaults( FL_IOPT * cntl )
-{
-    if ( ! fl_display )
-    {
-        M_err( "fl_get_defaults", "You forgot to call fl_initialize" );
-        exit( 1 );
-    }
-
-    *cntl = fli_cntl;
-}
-
-
 /***************************************
  * Convenience functions
  ***************************************/
@@ -335,7 +317,7 @@ fl_get_defaults( FL_IOPT * cntl )
 void
 fl_set_coordunit( int u )
 {
-    const char *cu = fli_get_vn_name( vn_coordunit, u );
+    const char * cu = fli_get_vn_name( vn_coordunit, u );
 
     if ( cu == NULL )
     {
@@ -368,6 +350,7 @@ void
 fl_set_scrollbar_type( int t )
 {
     fli_cntl.scrollbarType = t;
+
     if ( t == FL_NORMAL_SCROLLBAR )
         strcpy( OpSCBT, "normal" );
     else if ( t == FL_NICE_SCROLLBAR )
@@ -439,7 +422,7 @@ handle_applresdir( const char * rstr,
     strcpy( rbuf, rstr );
     for ( tok = strtok( rbuf, ":" ); tok; tok = strtok( 0, ":" ) )
     {
-        fl_snprintf( buf, sizeof buf,"%s/%s", tok,appclass );
+        fli_snprintf( buf, sizeof buf,"%s/%s", tok,appclass );
         M_info( "handle_applresdir", "Trying XAPPLRESDIR: %s", buf );
         if ( ( fdb = XrmGetFileDatabase( buf ) ) )
         {
@@ -458,8 +441,8 @@ handle_applresdir( const char * rstr,
 static void
 init_resource_database( const char *appclass )
 {
-    char buf[ FL_PATH_MAX + 127 ],
-         *rstr;
+    char   buf[ FL_PATH_MAX + 127 ],
+         * rstr;
     XrmDatabase fdb = 0;
 
     if ( ! fl_display )
@@ -482,7 +465,7 @@ init_resource_database( const char *appclass )
        The window resource manager for this display
     */
 
-    fl_snprintf( buf, sizeof buf, "DECW$SYSTEM_DEFAULTS:%s.DAT", appclass );
+    fli_snprintf( buf, sizeof buf, "DECW$SYSTEM_DEFAULTS:%s.DAT", appclass );
     M_info( "init_resource_database", "Trying Sys_default: %s", buf );
     if ( ( fdb = XrmGetFileDatabase( buf ) ) )
     {
@@ -490,7 +473,7 @@ init_resource_database( const char *appclass )
         M_warn( "init_resource_database", "System default %s loaded", buf );
     }
 
-    fl_snprintf( buf, sizeof buf, "DECW$USER_DEFAULTS:%s.DAT", appclass );
+    fli_snprintf( buf, sizeof buf, "DECW$USER_DEFAULTS:%s.DAT", appclass );
     M_info( "init_resource_database", "Trying User_default: %s", buf );
     if ( ( fdb = XrmGetFileDatabase( buf ) ) )
     {
@@ -498,7 +481,7 @@ init_resource_database( const char *appclass )
         M_warn( "init_resource_database", "System default %s loaded", buf );
     }
 
-    fl_snprintf( buf, sizeof buf, "DECW$USER_DEFAULTS:DECW$XDEFAULTS.DAT" );
+    fli_snprintf( buf, sizeof buf, "DECW$USER_DEFAULTS:DECW$XDEFAULTS.DAT" );
     M_info( "init_resource_database", "Trying Sys_default: %s", buf );
     if ( ( fdb = XrmGetFileDatabase( buf ) ) )
     {
@@ -517,7 +500,7 @@ init_resource_database( const char *appclass )
     }
 #else /* !VMS */
 
-    fl_snprintf( buf, sizeof buf, "/usr/lib/X11/app-defaults/%s", appclass );
+    fli_snprintf( buf, sizeof buf, "/usr/lib/X11/app-defaults/%s", appclass );
     M_info( "init_resource_database", "Trying Sys_default: %s", buf );
     if ( ( fdb = XrmGetFileDatabase( buf ) ) )
     {
@@ -544,11 +527,11 @@ init_resource_database( const char *appclass )
     }
     else
     {
-        /* try ~/.Xdefaults   */
+        /* Try ~/.Xdefaults   */
 
         if ( ( rstr = getenv( "HOME" ) ) )
         {
-            fl_snprintf( buf, sizeof buf,"%s/.Xdefaults", rstr );
+            fli_snprintf( buf, sizeof buf,"%s/.Xdefaults", rstr );
             M_info( "init_resource_database", "Trying %s", buf );
             if ( ( fdb = XrmGetFileDatabase( buf ) ) )
             {
@@ -558,7 +541,7 @@ init_resource_database( const char *appclass )
         }
     }
 
-    /* load file XENVIRONMENT */
+    /* Load file XENVIRONMENT */
 
     M_info( "init_resource_database",
             "Trying environment variable XEVIRONMENT" );
@@ -579,7 +562,7 @@ init_resource_database( const char *appclass )
         {
             int l;
 
-            fl_snprintf( buf, sizeof buf,"%s/.Xdefaults", rstr );
+            fli_snprintf( buf, sizeof buf,"%s/.Xdefaults", rstr );
             l = strlen( strcat( buf, "-" ) );
             gethostname( buf + l, sizeof buf - l );
             M_info( "init_resource_database", "Trying %s", buf );
@@ -607,7 +590,7 @@ init_resource_database( const char *appclass )
  ***************************************/
 
 static int
-is_true( const char *s )
+is_true( const char * s )
 {
     return    strncmp( s, "True", 4 ) == 0
            || strncmp( s, "true", 4 ) == 0
@@ -635,21 +618,22 @@ fl_get_resource( const char * rname,    /* resource name */
                  int          size )    /* buffer size if string */
 {
     XrmValue entry = { 0, NULL };
-    char *type = NULL;
+    char * type = NULL;
     char res_name[ 256 ]  = "",
          res_class[ 256 ] = "";
 
 
-    if ( ( ! rname || ! * rname ) && ( ! cname || ! *cname ) )
+    if ( ! ( rname && * rname ) && ! ( cname && *cname ) )
         return NULL;
 
     if ( rname && *rname )
-        fl_snprintf( res_name, sizeof res_name,"%s.%s", fl_app_name, rname );
-    else if ( cname && *cname )
-        fl_snprintf( res_class, sizeof res_class,"%s.%s", fl_app_class, cname );
+        fli_snprintf( res_name, sizeof res_name, "%s.%s", fl_app_name, rname );
+    else
+        fli_snprintf( res_class, sizeof res_class, "%s.%s",
+                     fl_app_class, cname );
 
     /* Just checking the return value of XrmGetResource() doesn't seem to
-       work (as it should, unless I completely mis-understand the man
+       work (as it should, unless I completely misunderstand the man
        page), and 'entry' seems to return the same data as on a previous
        call (despite the initialization!), but 'type' seems to get set to
        NULL in cases of failure to find the requested resource in the
@@ -673,23 +657,23 @@ fl_get_resource( const char * rname,    /* resource name */
     switch ( dtype )
     {
         case FL_SHORT:
-            *( short * ) val = atoi( entry.addr );
+            * ( short * ) val = strtol( entry.addr, NULL, 10 );
             break;
 
         case FL_INT:
-            *( int * ) val = atoi( entry.addr );
+            * ( int * ) val = strtol( entry.addr, NULL, 10 );
             break;
 
         case FL_BOOL:
-            *( int * ) val = is_true( entry.addr );
+            * ( int * ) val = is_true( entry.addr );
             break;
 
         case FL_LONG:
-            *( long * ) val = strtol( entry.addr, NULL, 0 );
+            * ( long * ) val = strtol( entry.addr, NULL, 10 );
             break;
 
         case FL_FLOAT:
-            * ( float * ) val = ( float ) atof( entry.addr );
+            * ( float * ) val = atof( entry.addr );
             break;
 
         case FL_STRING:
@@ -718,7 +702,7 @@ fl_set_resource( const char * str,
 {
     char res_name[ 256 ];
 
-    fl_snprintf( res_name, sizeof res_name, "%s.%s", fl_app_name, str );
+    fli_snprintf( res_name, sizeof res_name, "%s.%s", fl_app_name, str );
     XrmPutStringResource( &fldatabase, res_name, ( char * ) val );
 }
 
@@ -733,18 +717,18 @@ fli_init_resources( void )
     char res[ 256 ],
          cls[ 256 ],
          ores[ 256 ];
-    char *appname = fl_app_name,
-         *appclass = fl_app_class;
-    char *ori_appname = fl_ori_app_name;
+    char * appname = fl_app_name,
+         * appclass = fl_app_class;
+    char * ori_appname = fl_ori_app_name;
 
     /* internal resources need to be prefixed xform and XForm. need to
        generate for all names */
 
-    fl_snprintf( cls, sizeof cls, "%s.XForm", fl_app_class );
+    fli_snprintf( cls, sizeof cls, "%s.XForm", fl_app_class );
     fl_app_class = cls;
-    fl_snprintf( res, sizeof res, "%s.xform", fl_app_name );
+    fli_snprintf( res, sizeof res, "%s.xform", fl_app_name );
     fl_app_name = res;
-    fl_snprintf( ores, sizeof ores, "%s.xform", fl_ori_app_name );
+    fli_snprintf( ores, sizeof ores, "%s.xform", fl_ori_app_name );
     fl_ori_app_name = ores;
     fl_get_app_resources( internal_resources, Niopt );
 
@@ -761,8 +745,8 @@ fli_init_resources( void )
 }
 
 
-static int fl_argc;
-static char **fl_argv;
+static int fli_argc;
+static char ** fli_argv;
 
 
 /***************************************
@@ -777,13 +761,13 @@ dup_argv( char ** argv,
     if ( ! argv )
         return;
 
-    if ( ! fl_argv )
-        fl_argv = fl_malloc( ( n + 1 ) * sizeof *fl_argv );
+    if ( ! fli_argv )
+        fli_argv = fl_malloc( ( n + 1 ) * sizeof *fli_argv );
 
     for ( i = 0; i < n; i++ )
-        fl_argv[ i ] = fl_strdup( argv[ i ] );
+        fli_argv[ i ] = fl_strdup( argv[ i ] );
     
-    fl_argv[ i ] = NULL;
+    fli_argv[ i ] = NULL;
 }
 
 
@@ -791,10 +775,10 @@ dup_argv( char ** argv,
  ***************************************/
 
 char **
-fl_get_cmdline_args( int *n )
+fl_get_cmdline_args( int * n )
 {
-    *n = fl_argc;
-    return fl_argv;
+    *n = fli_argc;
+    return fli_argv;
 }
 
 
@@ -806,13 +790,13 @@ fli_free_cmdline_args( void )
 {
     size_t i;
 
-    if ( ! fl_argv )
+    if ( ! fli_argv )
         return;
 
-    for ( i = 0; fl_argv[ i ]; i++ )
-        fl_safe_free( fl_argv[ i ] );
+    for ( i = 0; fli_argv[ i ]; i++ )
+        fli_safe_free( fli_argv[ i ] );
 
-    fl_safe_free( fl_argv );
+    fli_safe_free( fli_argv );
 }
 
 
@@ -881,7 +865,12 @@ fli_init_context( void )
     if ( fli_context )
         return;
 
-    fli_context = fl_calloc( 1, sizeof *fli_context );
+    if ( ! ( fli_context = fl_calloc( 1, sizeof *fli_context ) ) )
+    {
+        M_err( "fli_init_context", "Running out of memory" );
+        exit( 1 );
+    }
+
     fli_context->io_rec        = NULL;
     fli_context->idle_rec      = NULL;
     fli_context->atclose       = NULL;
@@ -890,8 +879,8 @@ fli_init_context( void )
     fli_context->hscb          = FL_HOR_THIN_SCROLLBAR;
     fli_context->vscb          = FL_VERT_THIN_SCROLLBAR;
     fli_context->navigate_mask = ShiftMask;   /* to navigate input field */
-    fli_context->xim           = NULL;
-    fli_context->xic           = NULL;
+    fli_context->xim = NULL;
+    fli_context->xic = NULL;
 }
 
 
@@ -932,7 +921,7 @@ fl_initialize( int        * na,
         return fl_display;
     }
 
-    /* be paranoid */
+    /* Be paranoid */
 
     if ( ! na || ! *na )
     {
@@ -941,25 +930,13 @@ fl_initialize( int        * na,
         exit( 1 );
     }
 
-    /* Setting the locale was introduced in 0.89.5 (as far as I could figure
-       out) but never was officially documented. Since it can break programs
-       when run in a different locale then the one the original author used
-       (e.g. because reading of files with scanf() may not work anymore when,
-       due to the locale setting a ',' instead of a '.' as the decimal point is
-       expected) I have thrown it out. Jean-Marc and Angus already planned to
-       do that back in 2004 but obviously never got around to doing it.  JTT */
-
-#if 0
-    setlocale( LC_ALL, "" );
-#endif
-
     fli_internal_init( );
 
     XrmInitialize( );
 
     /* Save a copy of the command line for later WM hints */
 
-    fl_argc = *na;
+    fli_argc = *na;
     dup_argv( arg, *na );
 
     /* Get appname and class, but without any leading paths  */
@@ -989,16 +966,16 @@ fl_initialize( int        * na,
 
     /* Check version request */
 
-    fl_snprintf( disp_name, sizeof disp_name, "%s.fl_version", fl_app_name );
-    fl_snprintf( disp_cls, sizeof disp_cls, "%s.flversion", fl_app_name );
+    fli_snprintf( disp_name, sizeof disp_name, "%s.fl_version", fl_app_name );
+    fli_snprintf( disp_cls, sizeof disp_cls, "%s.flversion", fl_app_name );
 
     if ( XrmGetResource( cmddb, disp_name, disp_cls, &type, &xval ) )
         fli_print_version( 0 );
 
     /* Get the display name first before doing anything */
 
-    fl_snprintf( disp_name, sizeof disp_name, "%s.display", fl_ori_app_name );
-    fl_snprintf( disp_cls , sizeof disp_cls , "%s.Display", fl_app_class );
+    fli_snprintf( disp_name, sizeof disp_name, "%s.display", fl_ori_app_name );
+    fli_snprintf( disp_cls , sizeof disp_cls , "%s.Display", fl_app_class );
 
     *buf = '\0';
 
@@ -1008,14 +985,14 @@ fl_initialize( int        * na,
         buf[ sizeof buf - 1 ] = '\0';
     }
 
-    /* Open display and quit if failure */
+    /* Open display and quit on failure */
 
     if ( ! ( fl_display = XOpenDisplay( buf ) ) )
     {
         /* if no display is set, there is no guarantee that buf
            is long enough to contain the DISPLAY setting */
 
-        M_err( "fl_initialize", "%s: Can't open display %s", fl_argv[ 0 ],
+        M_err( "fl_initialize", "%s: Can't open display %s", fli_argv[ 0 ],
                XDisplayName( buf[ 0 ] ? buf : 0 ) );
         return 0;
     }
@@ -1026,15 +1003,15 @@ fl_initialize( int        * na,
     /* Get debug level settings since all error reporting will be controled
        by it */
 
-    fl_snprintf( disp_name, sizeof disp_name, "%s.fldebug", fl_app_name );
-    fl_snprintf( disp_cls , sizeof disp_cls , "%s.flDebug", fl_app_class );
+    fli_snprintf( disp_name, sizeof disp_name, "%s.fldebug", fl_app_name );
+    fli_snprintf( disp_cls , sizeof disp_cls , "%s.flDebug", fl_app_class );
     if ( XrmGetResource( cmddb, disp_name, disp_cls, &type, &xval ) )
         fli_set_debug_level( atoi( xval.addr ) );
 
     /* print help */
 
-    fl_snprintf( disp_name, sizeof disp_name, "%s.flhelp", fl_app_name );
-    fl_snprintf( disp_cls , sizeof disp_cls , "%s.flhelp", fl_app_class );
+    fli_snprintf( disp_name, sizeof disp_name, "%s.flhelp", fl_app_name );
+    fli_snprintf( disp_cls , sizeof disp_cls , "%s.flhelp", fl_app_class );
 
     if ( XrmGetResource( cmddb, disp_name, disp_cls, &type, &xval ) )
     {
@@ -1043,9 +1020,10 @@ fl_initialize( int        * na,
         while ( i < Ncopt )
         {
             if ( i == 0 )
-                fprintf( stderr, "%s: ", fl_argv[ 0 ] );
+                fprintf( stderr, "%s: ", fli_argv[ 0 ] );
             else
-                fprintf( stderr, "%*s  ", ( int ) strlen( fl_argv[ 0 ] ), "" );
+                fprintf( stderr, "%*s  ",
+                         ( int ) strlen( fli_argv[ 0 ] ), "" );
 
             fprintf( stderr, " %s", copt[ i ].option );
 
@@ -1060,9 +1038,11 @@ fl_initialize( int        * na,
 
     /* Check if -name is present */
 
-    fl_snprintf( disp_name, sizeof disp_name, "%s.name", fl_app_name );
-    fl_snprintf( disp_cls,  sizeof disp_cls , "%s.Name", fl_app_class );
+    fli_snprintf( disp_name, sizeof disp_name, "%s.name", fl_app_name );
+    fli_snprintf( disp_cls,  sizeof disp_cls , "%s.Name", fl_app_class );
+
     M_warn( "fl_initialize", "Trying display %s", disp_name );
+
     if ( XrmGetResource( cmddb, disp_name, disp_cls, &type, &xval ) )
     {
         fl_app_name = fl_strdup( xval.addr );
@@ -1082,7 +1062,7 @@ fl_initialize( int        * na,
 
     fli_init_resources( );
 
-    fli_cntl.vclass = fl_vclass_val( fli_cntl.vname );
+    fli_cntl.vclass = fli_vclass_val( fli_cntl.vname );
     fli_cntl.coordUnit = fli_get_vn_value( vn_coordunit, OpCoordUnit );
     if ( fli_cntl.coordUnit == -1 )
         fli_cntl.coordUnit = FL_COORD_PIXEL;
@@ -1094,7 +1074,7 @@ fl_initialize( int        * na,
         DumpD( debug );
         fprintf( stderr, "\tVisual:%s (%d)\n",
                  fli_cntl.vclass >= 0 ?
-                 fl_vclass_name( fli_cntl.vclass ) : "To be set",
+                 fli_vclass_name( fli_cntl.vclass ) : "To be set",
                  fli_cntl.vclass );
         DumpD( depth );
         DumpD( privateColormap );
@@ -1121,9 +1101,9 @@ fl_initialize( int        * na,
     fl_set_gamma( fli_cntl.rgamma, fli_cntl.ggamma, fli_cntl.bgamma );
 #endif
 
-    fl_set_ul_property( fli_cntl.ulPropWidth, fli_cntl.ulThickness );
+    fli_set_ul_property( fli_cntl.ulPropWidth, fli_cntl.ulThickness );
 
-    fli_cntl.vclass = fl_vclass_val( fli_cntl.vname );
+    fli_cntl.vclass = fli_vclass_val( fli_cntl.vname );
 
     /* Get the current keyboard state */
 
@@ -1264,14 +1244,14 @@ fl_initialize( int        * na,
     fli_context->ext_request_size -= 8;
     fli_context->tooltip_time = 600;
 
-    fl_add_io_callback( ConnectionNumber( fl_display ), FL_READ, 0, 0 );
+    fl_add_io_callback( ConnectionNumber( fl_display ), FL_READ, NULL, NULL );
 
-    /* has to be here, otherwise fl_add_symbol won't be able to replace the
+    /* Has to be here, otherwise fl_add_symbol won't be able to replace the
        built-in */
 
     fli_init_symbols( );
 
-    /* hang the database on the display so application can get it */
+    /* Hang the database on the display so application can get it */
 
     XrmSetDatabase( fl_display, fldatabase );
 
@@ -1286,7 +1266,6 @@ fl_initialize( int        * na,
 /***************************************
  * Cleanup everything. At the moment, only need to restore the keyboard
  * settings and deallocate memory used for the object and event queue etc.
- * This routine is registered as an atexit in fl_initialize in flresource.
  ***************************************/
 
 void
@@ -1300,8 +1279,8 @@ fl_finish( void )
     XChangeKeyboardControl( flx->display, fli_keybdmask,
                             &fli_keybdcontrol );
 
-    fl_remove_all_signal_callbacks( );
-    fl_remove_all_timeouts( );
+    fli_remove_all_signal_callbacks( );
+    fli_remove_all_timeouts( );
 
     /* Get rid of all forms, first hide all of them */
 
@@ -1370,7 +1349,13 @@ fl_finish( void )
     }
 #endif
 
-    fl_safe_free( fli_context );
+    if ( fli_context )
+        while ( fli_context->io_rec )
+            fl_remove_io_callback( fli_context->io_rec->source,
+                                   fli_context->io_rec->mask,
+                                   fli_context->io_rec->callback );
+
+    fli_safe_free( fli_context );
 
     /* Close the display */
 
@@ -1380,11 +1365,19 @@ fl_finish( void )
 
 
 /***************************************
- * Find out about  virtual root. Taken from XFaq
+ * Find out about virtual root. Taken from XFaq
  ***************************************/
 
 #include <X11/Xatom.h>
 
+static int
+xerror_handler( Display     * d  FL_UNUSED_ARG,
+                XErrorEvent * xev )
+{
+    return xev->error_code;
+}
+
+
 static Window
 fli_GetVRoot( Display * dpy,
               int       scr )
@@ -1396,11 +1389,21 @@ fli_GetVRoot( Display * dpy,
     Window root = RootWindow( dpy, scr );
     Atom __SWM_VROOT = None;
     unsigned int i;
+    int ( * oldhandler )( Display *, XErrorEvent * );
 
     __SWM_VROOT = XInternAtom( dpy, "__SWM_VROOT", False );
     XQueryTree( dpy, root, &rootReturn, &parentReturn, &children,
                 &numChildren );
 
+    /* For some reasons XQueryTree sometimes seems to return an invalid window
+       in the list of children (perhaps the window  vanishes between the call
+       of XQueryTree() and the subsequent call of XGetWindowProperty()?). To
+       avoid a program using XForms aborting with a strange X error message on
+       start-up, we catch these errors with a temporary error handler (which
+       gets reset before leaving the function). */
+
+    oldhandler = XSetErrorHandler( xerror_handler );
+
     for ( i = 0; i < numChildren; i++ )
     {
         Atom actual_type;
@@ -1412,6 +1415,7 @@ fli_GetVRoot( Display * dpy,
            ( unsigned char ** ) ( void * ) &newRoot
            is there to avoid a spurious  GCC warning         JTT */
 
+        children[ i ] = 0;
         if (    XGetWindowProperty( dpy, children[ i ], __SWM_VROOT, 0, 1,
                                     False, XA_WINDOW, &actual_type,
                                     &actual_format, &nitems, &bytesafter,
@@ -1424,6 +1428,7 @@ fli_GetVRoot( Display * dpy,
         }
     }
 
+    XSetErrorHandler( oldhandler );
     XFree( ( char * ) children );
     return root;
 }
